// Copyright (C) 2020 ~ 2021 Uniontech Software Technology Co.,Ltd.
// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "constant.h"

const int Constant::RECTANGLE_PADDING = 24;
const int Constant::RECTANGLE_RADIUS = 8;
const int Constant::RECTANGLE_FONT_SIZE = 11;
