/****************************************************************************
** Meta object code from reading C++ file 'subtoolwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "widgets/subtoolwidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'subtoolwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13SubToolWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto SubToolWidget::qt_create_metaobjectdata<qt_meta_tag_ZN13SubToolWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SubToolWidget",
        "keyBoardButtonClicked",
        "",
        "checked",
        "mouseBoardButtonClicked",
        "mouseShowButtonClicked",
        "cameraActionChecked",
        "changeShotToolFunc",
        "func",
        "saveMethodChanged",
        "SaveAction",
        "saveact",
        "path",
        "saveToClipBoard",
        "isClip",
        "shotOptionMenuShown",
        "switchContent",
        "shapeType",
        "setRecordButtonDisable",
        "setRecordLaunchMode",
        "funType",
        "setVideoButtonInitFromSub",
        "shapeClickedFromWidget",
        "shape",
        "setMicroPhoneEnable",
        "status",
        "setCameraDeviceEnable"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'keyBoardButtonClicked'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'mouseBoardButtonClicked'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'mouseShowButtonClicked'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'cameraActionChecked'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'changeShotToolFunc'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'saveMethodChanged'
        QtMocHelpers::SignalData<void(SaveAction, const QString)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::QString, 12 },
        }}),
        // Signal 'saveToClipBoard'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'shotOptionMenuShown'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'switchContent'
        QtMocHelpers::SlotData<void(QString)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'setRecordButtonDisable'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setRecordLaunchMode'
        QtMocHelpers::SlotData<void(const unsigned int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 20 },
        }}),
        // Slot 'setVideoButtonInitFromSub'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shapeClickedFromWidget'
        QtMocHelpers::SlotData<void(QString)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'setMicroPhoneEnable'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setCameraDeviceEnable'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SubToolWidget, qt_meta_tag_ZN13SubToolWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SubToolWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<DStackedWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SubToolWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SubToolWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13SubToolWidgetE_t>.metaTypes,
    nullptr
} };

void SubToolWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SubToolWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->keyBoardButtonClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->mouseBoardButtonClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->mouseShowButtonClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->cameraActionChecked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->changeShotToolFunc((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->saveMethodChanged((*reinterpret_cast<std::add_pointer_t<SaveAction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->saveToClipBoard((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->shotOptionMenuShown(); break;
        case 8: _t->switchContent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->setRecordButtonDisable(); break;
        case 10: _t->setRecordLaunchMode((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 11: _t->setVideoButtonInitFromSub(); break;
        case 12: _t->shapeClickedFromWidget((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->setMicroPhoneEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setCameraDeviceEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SaveAction >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SubToolWidget::*)(bool )>(_a, &SubToolWidget::keyBoardButtonClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubToolWidget::*)(bool )>(_a, &SubToolWidget::mouseBoardButtonClicked, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubToolWidget::*)(bool )>(_a, &SubToolWidget::mouseShowButtonClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubToolWidget::*)(bool )>(_a, &SubToolWidget::cameraActionChecked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubToolWidget::*)(const QString & )>(_a, &SubToolWidget::changeShotToolFunc, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubToolWidget::*)(SaveAction , const QString )>(_a, &SubToolWidget::saveMethodChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubToolWidget::*)(bool )>(_a, &SubToolWidget::saveToClipBoard, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubToolWidget::*)()>(_a, &SubToolWidget::shotOptionMenuShown, 7))
            return;
    }
}

const QMetaObject *SubToolWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SubToolWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SubToolWidgetE_t>.strings))
        return static_cast<void*>(this);
    return DStackedWidget::qt_metacast(_clname);
}

int SubToolWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DStackedWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void SubToolWidget::keyBoardButtonClicked(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SubToolWidget::mouseBoardButtonClicked(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SubToolWidget::mouseShowButtonClicked(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SubToolWidget::cameraActionChecked(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void SubToolWidget::changeShotToolFunc(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void SubToolWidget::saveMethodChanged(SaveAction _t1, const QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void SubToolWidget::saveToClipBoard(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void SubToolWidget::shotOptionMenuShown()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
