/****************************************************************************
** Meta object code from reading C++ file 'shapeswidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "widgets/shapeswidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'shapeswidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12ShapesWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto ShapesWidget::qt_create_metaobjectdata<qt_meta_tag_ZN12ShapesWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ShapesWidget",
        "reloadEffectImg",
        "",
        "effect",
        "radius",
        "requestScreenshot",
        "shapePressed",
        "shape",
        "saveBtnPressed",
        "SaveAction",
        "action",
        "requestExit",
        "menuNoFocus",
        "saveFromMenu",
        "closeFromMenu",
        "shapeClicked",
        "setShapesUndo",
        "status",
        "updateSelectedShape",
        "group",
        "key",
        "index",
        "setCurrentShape",
        "shapeType",
        "clearSelected",
        "setAllTextEditReadOnly",
        "setNoChangedTextEditRemove",
        "saveActionTriggered",
        "handleDrag",
        "QPointF",
        "oldPoint",
        "newPoint",
        "handleRotate",
        "pos",
        "handleResize",
        "clickedOnShapes",
        "clickedOnRect",
        "FourPoints",
        "rectPoints",
        "isBlurMosaic",
        "clickedOnEllipse",
        "mainPoints",
        "clickedOnArrow",
        "QList<QPointF>",
        "points",
        "clickedOnLine",
        "clickedOnText",
        "rotateOnPoint",
        "hoverOnShapes",
        "Toolshape",
        "toolShape",
        "hoverOnRect",
        "isTextBorder",
        "hoverOnEllipse",
        "hoverOnArrow",
        "hoverOnLine",
        "hoverOnText",
        "textIndex",
        "hoverOnRotatePoint",
        "textEditIsReadOnly",
        "undoDrawShapes",
        "undoAllDrawShapes",
        "isInUndoBtn",
        "isUndo",
        "deleteCurrentShape",
        "microAdjust",
        "direction",
        "setShiftKeyPressed",
        "isShift",
        "updateCursorShape",
        "menuSaveSlot",
        "menuCloseSlot",
        "setGlobalRect",
        "QRect",
        "rect",
        "paintImage",
        "QImage&",
        "image",
        "isExistsText"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'reloadEffectImg'
        QtMocHelpers::SignalData<void(QString, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'requestScreenshot'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shapePressed'
        QtMocHelpers::SignalData<void(QString)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'saveBtnPressed'
        QtMocHelpers::SignalData<void(SaveAction)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'requestExit'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'menuNoFocus'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saveFromMenu'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closeFromMenu'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shapeClicked'
        QtMocHelpers::SignalData<void(QString)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'setShapesUndo'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'updateSelectedShape'
        QtMocHelpers::SlotData<void(const QString &, const QString &, int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 }, { QMetaType::QString, 20 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'setCurrentShape'
        QtMocHelpers::SlotData<void(QString)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'clearSelected'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setAllTextEditReadOnly'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setNoChangedTextEditRemove'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveActionTriggered'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleDrag'
        QtMocHelpers::SlotData<void(QPointF, QPointF)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { 0x80000000 | 29, 31 },
        }}),
        // Slot 'handleRotate'
        QtMocHelpers::SlotData<void(QPointF)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 33 },
        }}),
        // Slot 'handleResize'
        QtMocHelpers::SlotData<void(QPointF, int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 33 }, { QMetaType::Int, 20 },
        }}),
        // Slot 'clickedOnShapes'
        QtMocHelpers::SlotData<bool(QPointF)>(35, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 29, 33 },
        }}),
        // Slot 'clickedOnRect'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF, bool)>(36, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 37, 38 }, { 0x80000000 | 29, 33 }, { QMetaType::Bool, 39 },
        }}),
        // Slot 'clickedOnRect'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF)>(36, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 37, 38 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'clickedOnEllipse'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF, bool)>(40, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 37, 41 }, { 0x80000000 | 29, 33 }, { QMetaType::Bool, 39 },
        }}),
        // Slot 'clickedOnEllipse'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF)>(40, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 37, 41 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'clickedOnArrow'
        QtMocHelpers::SlotData<bool(QList<QPointF>, QPointF)>(42, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 43, 44 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'clickedOnLine'
        QtMocHelpers::SlotData<bool(FourPoints, QList<QPointF>, QPointF)>(45, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 37, 41 }, { 0x80000000 | 43, 44 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'clickedOnText'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF)>(46, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 37, 41 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'rotateOnPoint'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF)>(47, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 37, 41 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'hoverOnShapes'
        QtMocHelpers::SlotData<bool(Toolshape, QPointF)>(48, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 49, 50 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'hoverOnRect'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF, bool)>(51, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 37, 38 }, { 0x80000000 | 29, 33 }, { QMetaType::Bool, 52 },
        }}),
        // Slot 'hoverOnRect'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF)>(51, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 37, 38 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'hoverOnEllipse'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF)>(53, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 37, 41 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'hoverOnArrow'
        QtMocHelpers::SlotData<bool(QList<QPointF>, QPointF)>(54, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 43, 44 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'hoverOnLine'
        QtMocHelpers::SlotData<bool(FourPoints, QList<QPointF>, QPointF)>(55, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 37, 41 }, { 0x80000000 | 43, 44 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'hoverOnText'
        QtMocHelpers::SlotData<bool(int, FourPoints, QPointF)>(56, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 57 }, { 0x80000000 | 37, 41 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'hoverOnRotatePoint'
        QtMocHelpers::SlotData<bool(FourPoints, QPointF)>(58, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 37, 41 }, { 0x80000000 | 29, 33 },
        }}),
        // Slot 'textEditIsReadOnly'
        QtMocHelpers::SlotData<bool()>(59, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'undoDrawShapes'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'undoAllDrawShapes'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'isInUndoBtn'
        QtMocHelpers::SlotData<void(bool)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'deleteCurrentShape'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'microAdjust'
        QtMocHelpers::SlotData<void(QString)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 66 },
        }}),
        // Slot 'setShiftKeyPressed'
        QtMocHelpers::SlotData<void(bool)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 68 },
        }}),
        // Slot 'updateCursorShape'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuSaveSlot'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'menuCloseSlot'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setGlobalRect'
        QtMocHelpers::SlotData<void(QRect)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 73, 74 },
        }}),
        // Slot 'paintImage'
        QtMocHelpers::SlotData<void(QImage &)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 76, 77 },
        }}),
        // Slot 'isExistsText'
        QtMocHelpers::SlotData<bool()>(78, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ShapesWidget, qt_meta_tag_ZN12ShapesWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ShapesWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<DFrame::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ShapesWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ShapesWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12ShapesWidgetE_t>.metaTypes,
    nullptr
} };

void ShapesWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShapesWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->reloadEffectImg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->requestScreenshot(); break;
        case 2: _t->shapePressed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->saveBtnPressed((*reinterpret_cast<std::add_pointer_t<SaveAction>>(_a[1]))); break;
        case 4: _t->requestExit(); break;
        case 5: _t->menuNoFocus(); break;
        case 6: _t->saveFromMenu(); break;
        case 7: _t->closeFromMenu(); break;
        case 8: _t->shapeClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->setShapesUndo((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->updateSelectedShape((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 11: _t->setCurrentShape((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->clearSelected(); break;
        case 13: _t->setAllTextEditReadOnly(); break;
        case 14: _t->setNoChangedTextEditRemove(); break;
        case 15: _t->saveActionTriggered(); break;
        case 16: _t->handleDrag((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 17: _t->handleRotate((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 18: _t->handleResize((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 19: { bool _r = _t->clickedOnShapes((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->clickedOnRect((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->clickedOnRect((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->clickedOnEllipse((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->clickedOnEllipse((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->clickedOnArrow((*reinterpret_cast<std::add_pointer_t<QList<QPointF>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->clickedOnLine((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QPointF>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->clickedOnText((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->rotateOnPoint((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->hoverOnShapes((*reinterpret_cast<std::add_pointer_t<Toolshape>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->hoverOnRect((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->hoverOnRect((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->hoverOnEllipse((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->hoverOnArrow((*reinterpret_cast<std::add_pointer_t<QList<QPointF>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->hoverOnLine((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QPointF>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->hoverOnText((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->hoverOnRotatePoint((*reinterpret_cast<std::add_pointer_t<FourPoints>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 36: { bool _r = _t->textEditIsReadOnly();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 37: _t->undoDrawShapes(); break;
        case 38: _t->undoAllDrawShapes(); break;
        case 39: _t->isInUndoBtn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->deleteCurrentShape(); break;
        case 41: _t->microAdjust((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->setShiftKeyPressed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->updateCursorShape(); break;
        case 44: _t->menuSaveSlot(); break;
        case 45: _t->menuCloseSlot(); break;
        case 46: _t->setGlobalRect((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1]))); break;
        case 47: _t->paintImage((*reinterpret_cast<std::add_pointer_t<QImage&>>(_a[1]))); break;
        case 48: { bool _r = _t->isExistsText();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)(QString , int )>(_a, &ShapesWidget::reloadEffectImg, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)()>(_a, &ShapesWidget::requestScreenshot, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)(QString )>(_a, &ShapesWidget::shapePressed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)(SaveAction )>(_a, &ShapesWidget::saveBtnPressed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)()>(_a, &ShapesWidget::requestExit, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)()>(_a, &ShapesWidget::menuNoFocus, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)()>(_a, &ShapesWidget::saveFromMenu, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)()>(_a, &ShapesWidget::closeFromMenu, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)(QString )>(_a, &ShapesWidget::shapeClicked, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShapesWidget::*)(bool )>(_a, &ShapesWidget::setShapesUndo, 9))
            return;
    }
}

const QMetaObject *ShapesWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ShapesWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ShapesWidgetE_t>.strings))
        return static_cast<void*>(this);
    return DFrame::qt_metacast(_clname);
}

int ShapesWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 49)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 49;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 49)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 49;
    }
    return _id;
}

// SIGNAL 0
void ShapesWidget::reloadEffectImg(QString _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void ShapesWidget::requestScreenshot()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ShapesWidget::shapePressed(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ShapesWidget::saveBtnPressed(SaveAction _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ShapesWidget::requestExit()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ShapesWidget::menuNoFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ShapesWidget::saveFromMenu()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void ShapesWidget::closeFromMenu()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void ShapesWidget::shapeClicked(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void ShapesWidget::setShapesUndo(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
