/****************************************************************************
** Meta object code from reading C++ file 'dbusscreenshotservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "dbusservice/dbusscreenshotservice.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusscreenshotservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21DBusScreenshotServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto DBusScreenshotService::qt_create_metaobjectdata<qt_meta_tag_ZN21DBusScreenshotServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DBusScreenshotService",
        "D-Bus Interface",
        "com.deepin.Screenshot",
        "D-Bus Introspection",
        "  <interface name=\"com.deepin.Screenshot\">\n    <method name=\"S"
        "tartScreenshot\"/>\n    <method name=\"DelayScreenshot\">\n      <"
        "arg direction=\"in\" type=\"x\"/>\n   </method>\n    <method name="
        "\"NoNotifyScreenshot\"/>\n    <method name=\"TopWindowScreenshot\""
        "/>\n    <method name=\"FullscreenScreenshot\"/>\n    <method name="
        "\"FullScreenRecord\">\n      <arg direction=\"in\" type=\"s\"/>\n "
        "   </method>\n    <method name=\"OcrScreenshot\"/>\n    <method na"
        "me=\"ScrollScreenshot\"/>\n    <method name=\"SavePathScreenshot\""
        ">\n      <arg direction=\"in\" type=\"s\"/>\n    </method>\n    <m"
        "ethod name=\"StartScreenshotFor3rd\">\n      <arg direction=\"in\""
        " type=\"s\"/>\n    </method>\n    <signal name=\"Done\">\n      <a"
        "rg type=\"s\"/>\n    </signal>\n  </interface>\n",
        "Done",
        "",
        "in0",
        "StartScreenshot",
        "DelayScreenshot",
        "NoNotifyScreenshot",
        "OcrScreenshot",
        "ScrollScreenshot",
        "TopWindowScreenshot",
        "FullscreenScreenshot",
        "SavePathScreenshot",
        "StartScreenshotFor3rd",
        "FullScreenRecord"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Done'
        QtMocHelpers::SignalData<void(const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'StartScreenshot'
        QtMocHelpers::SlotData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'DelayScreenshot'
        QtMocHelpers::SlotData<void(qlonglong)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 },
        }}),
        // Slot 'NoNotifyScreenshot'
        QtMocHelpers::SlotData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OcrScreenshot'
        QtMocHelpers::SlotData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ScrollScreenshot'
        QtMocHelpers::SlotData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'TopWindowScreenshot'
        QtMocHelpers::SlotData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'FullscreenScreenshot'
        QtMocHelpers::SlotData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SavePathScreenshot'
        QtMocHelpers::SlotData<void(const QString &)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'StartScreenshotFor3rd'
        QtMocHelpers::SlotData<void(const QString &)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'FullScreenRecord'
        QtMocHelpers::SlotData<void(const QString &)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<DBusScreenshotService, qt_meta_tag_ZN21DBusScreenshotServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DBusScreenshotService::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DBusScreenshotServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DBusScreenshotServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21DBusScreenshotServiceE_t>.metaTypes,
    nullptr
} };

void DBusScreenshotService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DBusScreenshotService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Done((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->StartScreenshot(); break;
        case 2: _t->DelayScreenshot((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1]))); break;
        case 3: _t->NoNotifyScreenshot(); break;
        case 4: _t->OcrScreenshot(); break;
        case 5: _t->ScrollScreenshot(); break;
        case 6: _t->TopWindowScreenshot(); break;
        case 7: _t->FullscreenScreenshot(); break;
        case 8: _t->SavePathScreenshot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->StartScreenshotFor3rd((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->FullScreenRecord((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DBusScreenshotService::*)(const QString & )>(_a, &DBusScreenshotService::Done, 0))
            return;
    }
}

const QMetaObject *DBusScreenshotService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusScreenshotService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DBusScreenshotServiceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int DBusScreenshotService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void DBusScreenshotService::Done(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
