/****************************************************************************
** Meta object code from reading C++ file 'screenshot.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "screenshot.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'screenshot.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10ScreenshotE_t {};
} // unnamed namespace

template <> constexpr inline auto Screenshot::qt_create_metaobjectdata<qt_meta_tag_ZN10ScreenshotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Screenshot",
        "D-Bus Interface",
        "com.deepin.ScreenRecorder",
        "RecorderState",
        "",
        "isStart",
        "startScreenshot",
        "delayScreenshot",
        "num",
        "fullscreenScreenshot",
        "topWindowScreenshot",
        "noNotifyScreenshot",
        "OcrScreenshot",
        "ScrollScreenshot",
        "savePathScreenshot",
        "path",
        "startScreenshotFor3rd",
        "initLaunchMode",
        "launchmode",
        "fullScreenRecord",
        "fileName",
        "stopRecord",
        "getRecorderNormalIcon"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'RecorderState'
        QtMocHelpers::SignalData<void(const bool)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'startScreenshot'
        QtMocHelpers::SlotData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'delayScreenshot'
        QtMocHelpers::SlotData<void(double)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'fullscreenScreenshot'
        QtMocHelpers::SlotData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'topWindowScreenshot'
        QtMocHelpers::SlotData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'noNotifyScreenshot'
        QtMocHelpers::SlotData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OcrScreenshot'
        QtMocHelpers::SlotData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ScrollScreenshot'
        QtMocHelpers::SlotData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'savePathScreenshot'
        QtMocHelpers::SlotData<void(const QString &)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'startScreenshotFor3rd'
        QtMocHelpers::SlotData<void(const QString &)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'initLaunchMode'
        QtMocHelpers::SlotData<void(const QString &)>(17, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'fullScreenRecord'
        QtMocHelpers::SlotData<void(QString)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'fullScreenRecord'
        QtMocHelpers::SlotData<void()>(19, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'stopRecord'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'getRecorderNormalIcon'
        QtMocHelpers::SlotData<QString()>(22, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Screenshot, qt_meta_tag_ZN10ScreenshotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Screenshot::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ScreenshotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ScreenshotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10ScreenshotE_t>.metaTypes,
    nullptr
} };

void Screenshot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Screenshot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->RecorderState((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->startScreenshot(); break;
        case 2: _t->delayScreenshot((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->fullscreenScreenshot(); break;
        case 4: _t->topWindowScreenshot(); break;
        case 5: _t->noNotifyScreenshot(); break;
        case 6: _t->OcrScreenshot(); break;
        case 7: _t->ScrollScreenshot(); break;
        case 8: _t->savePathScreenshot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->startScreenshotFor3rd((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->initLaunchMode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->fullScreenRecord((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->fullScreenRecord(); break;
        case 13: _t->stopRecord(); break;
        case 14: { QString _r = _t->getRecorderNormalIcon();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Screenshot::*)(const bool )>(_a, &Screenshot::RecorderState, 0))
            return;
    }
}

const QMetaObject *Screenshot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Screenshot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ScreenshotE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Screenshot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Screenshot::RecorderState(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
