/****************************************************************************
** Meta object code from reading C++ file 'capture.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "capture.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'capture.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20TreelandCaptureFrameE_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandCaptureFrame::qt_create_metaobjectdata<qt_meta_tag_ZN20TreelandCaptureFrameE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandCaptureFrame",
        "ready",
        "",
        "QImage",
        "image",
        "failed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ready'
        QtMocHelpers::SignalData<void(QImage)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'failed'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreelandCaptureFrame, qt_meta_tag_ZN20TreelandCaptureFrameE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreelandCaptureFrame::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TreelandCaptureFrameE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TreelandCaptureFrameE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20TreelandCaptureFrameE_t>.metaTypes,
    nullptr
} };

void TreelandCaptureFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandCaptureFrame *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ready((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 1: _t->failed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureFrame::*)(QImage )>(_a, &TreelandCaptureFrame::ready, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureFrame::*)()>(_a, &TreelandCaptureFrame::failed, 1))
            return;
    }
}

const QMetaObject *TreelandCaptureFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandCaptureFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20TreelandCaptureFrameE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_capture_frame_v1"))
        return static_cast< QtWayland::treeland_capture_frame_v1*>(this);
    return QObject::qt_metacast(_clname);
}

int TreelandCaptureFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void TreelandCaptureFrame::ready(QImage _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TreelandCaptureFrame::failed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN22TreelandCaptureSessionE_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandCaptureSession::qt_create_metaobjectdata<qt_meta_tag_ZN22TreelandCaptureSessionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandCaptureSession",
        "invalid",
        "",
        "ready",
        "startedChanged",
        "started"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'invalid'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ready'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startedChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'started'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreelandCaptureSession, qt_meta_tag_ZN22TreelandCaptureSessionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreelandCaptureSession::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TreelandCaptureSessionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TreelandCaptureSessionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22TreelandCaptureSessionE_t>.metaTypes,
    nullptr
} };

void TreelandCaptureSession::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandCaptureSession *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->invalid(); break;
        case 1: _t->ready(); break;
        case 2: _t->startedChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureSession::*)()>(_a, &TreelandCaptureSession::invalid, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureSession::*)()>(_a, &TreelandCaptureSession::ready, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureSession::*)()>(_a, &TreelandCaptureSession::startedChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->started(); break;
        default: break;
        }
    }
}

const QMetaObject *TreelandCaptureSession::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandCaptureSession::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TreelandCaptureSessionE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_capture_session_v1"))
        return static_cast< QtWayland::treeland_capture_session_v1*>(this);
    return QObject::qt_metacast(_clname);
}

int TreelandCaptureSession::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void TreelandCaptureSession::invalid()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TreelandCaptureSession::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TreelandCaptureSession::startedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN22TreelandCaptureContextE_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandCaptureContext::qt_create_metaobjectdata<qt_meta_tag_ZN22TreelandCaptureContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandCaptureContext",
        "sourceReady",
        "",
        "QRect",
        "region",
        "uint32_t",
        "sourceType",
        "sourceFailed",
        "reason",
        "frameChanged",
        "sessionChanged",
        "captureRegionChanged",
        "frame",
        "TreelandCaptureFrame*",
        "session",
        "TreelandCaptureSession*",
        "captureRegion",
        "QRectF"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sourceReady'
        QtMocHelpers::SignalData<void(QRect, uint32_t)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'sourceFailed'
        QtMocHelpers::SignalData<void(uint32_t)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 8 },
        }}),
        // Signal 'frameChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sessionChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'captureRegionChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'frame'
        QtMocHelpers::PropertyData<TreelandCaptureFrame*>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 2),
        // property 'session'
        QtMocHelpers::PropertyData<TreelandCaptureSession*>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 3),
        // property 'captureRegion'
        QtMocHelpers::PropertyData<QRectF>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreelandCaptureContext, qt_meta_tag_ZN22TreelandCaptureContextE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreelandCaptureContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TreelandCaptureContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TreelandCaptureContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22TreelandCaptureContextE_t>.metaTypes,
    nullptr
} };

void TreelandCaptureContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandCaptureContext *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sourceReady((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[2]))); break;
        case 1: _t->sourceFailed((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 2: _t->frameChanged(); break;
        case 3: _t->sessionChanged(); break;
        case 4: _t->captureRegionChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureContext::*)(QRect , uint32_t )>(_a, &TreelandCaptureContext::sourceReady, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureContext::*)(uint32_t )>(_a, &TreelandCaptureContext::sourceFailed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureContext::*)()>(_a, &TreelandCaptureContext::frameChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureContext::*)()>(_a, &TreelandCaptureContext::sessionChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureContext::*)()>(_a, &TreelandCaptureContext::captureRegionChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TreelandCaptureFrame* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TreelandCaptureSession* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<TreelandCaptureFrame**>(_v) = _t->frame(); break;
        case 1: *reinterpret_cast<TreelandCaptureSession**>(_v) = _t->session(); break;
        case 2: *reinterpret_cast<QRectF*>(_v) = _t->captureRegion(); break;
        default: break;
        }
    }
}

const QMetaObject *TreelandCaptureContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandCaptureContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TreelandCaptureContextE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_capture_context_v1"))
        return static_cast< QtWayland::treeland_capture_context_v1*>(this);
    return QObject::qt_metacast(_clname);
}

int TreelandCaptureContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void TreelandCaptureContext::sourceReady(QRect _t1, uint32_t _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void TreelandCaptureContext::sourceFailed(uint32_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TreelandCaptureContext::frameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TreelandCaptureContext::sessionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TreelandCaptureContext::captureRegionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
namespace {
struct qt_meta_tag_ZN22TreelandCaptureManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandCaptureManager::qt_create_metaobjectdata<qt_meta_tag_ZN22TreelandCaptureManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandCaptureManager",
        "contextChanged",
        "",
        "recordChanged",
        "finishSelect",
        "recordStartedChanged",
        "context",
        "TreelandCaptureContext*",
        "record",
        "recordStarted"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'contextChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'recordChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finishSelect'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'recordStartedChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'context'
        QtMocHelpers::PropertyData<TreelandCaptureContext*>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
        // property 'record'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'recordStarted'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreelandCaptureManager, qt_meta_tag_ZN22TreelandCaptureManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreelandCaptureManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandClientExtensionTemplate<TreelandCaptureManager>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TreelandCaptureManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TreelandCaptureManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22TreelandCaptureManagerE_t>.metaTypes,
    nullptr
} };

void TreelandCaptureManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandCaptureManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->contextChanged(); break;
        case 1: _t->recordChanged(); break;
        case 2: _t->finishSelect(); break;
        case 3: _t->recordStartedChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureManager::*)()>(_a, &TreelandCaptureManager::contextChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureManager::*)()>(_a, &TreelandCaptureManager::recordChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureManager::*)()>(_a, &TreelandCaptureManager::finishSelect, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandCaptureManager::*)()>(_a, &TreelandCaptureManager::recordStartedChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TreelandCaptureContext* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<TreelandCaptureContext**>(_v) = _t->context(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->record(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->recordStarted(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setRecord(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *TreelandCaptureManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandCaptureManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TreelandCaptureManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_capture_manager_v1"))
        return static_cast< QtWayland::treeland_capture_manager_v1*>(this);
    return QWaylandClientExtensionTemplate<TreelandCaptureManager>::qt_metacast(_clname);
}

int TreelandCaptureManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandClientExtensionTemplate<TreelandCaptureManager>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void TreelandCaptureManager::contextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TreelandCaptureManager::recordChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TreelandCaptureManager::finishSelect()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TreelandCaptureManager::recordStartedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
